import { nanoSQLMemoryIndex } from '@nano-sql/core/lib/adapters/memoryIndex';
import { SQLiteAbstract } from '@nano-sql/core/lib/adapters/webSQL';
import * as NSSqlite from '../sqlite';
export class NativeSQLite extends nanoSQLMemoryIndex {
    constructor(filePath, options) {
        super(false, true);
        this.filePath = filePath;
        this.options = options;
        this.plugin = {
            name: 'Akylas NativeScript SQLite Adapter',
            version: 2.37,
        };
        this._ai = {};
        this._query = this._query.bind(this);
        this._tables = {};
        this._sqlite = SQLiteAbstract(this._query, 500);
    }
    connect(id, complete, error) {
        this._id = id;
        try {
            this._db = NSSqlite.openOrCreate(this.filePath && this.filePath.length ? this.filePath : id, this.options || {});
            this._sqlite.createAI(() => {
                complete();
            }, error);
        }
        catch (err) {
            error(err);
        }
    }
    createTable(tableName, tableData, complete, error) {
        this._tables[tableName] = tableData;
        this._sqlite.createTable(tableName, tableData, this._ai, complete, error);
    }
    async _query(allowWrite, sql, args, onRow, complete, error) {
        if (allowWrite) {
            try {
                await this._db.execute(sql, args);
                complete();
            }
            catch (err) {
                error(err);
            }
        }
        else {
            try {
                let count = 0;
                await this._db.each(sql, args, (err, row) => {
                    if (err) {
                        error(err);
                        return;
                    }
                    onRow(row, count);
                    count++;
                }, (err) => {
                    if (err) {
                        error(err);
                        return;
                    }
                    complete();
                });
            }
            catch (err) {
                error(err);
            }
        }
    }
    dropTable(table, complete, error) {
        this._sqlite.dropTable(table, complete, error);
    }
    disconnect(complete, error) {
        complete();
    }
    write(table, pk, row, complete, error) {
        this._sqlite.write(this._tables[table].pkType, this._tables[table].pkCol, table, pk, row, this._tables[table].ai, this._ai, complete, error);
    }
    read(table, pk, complete, error) {
        this._sqlite.read(table, pk, complete, error);
    }
    delete(table, pk, complete, error) {
        this._sqlite.remove(table, pk, complete, error);
    }
    readMulti(table, type, offsetOrLow, limitOrHigh, reverse, onRow, complete, error) {
        this._sqlite.readMulti(table, type, offsetOrLow, limitOrHigh, reverse, onRow, complete, error);
    }
    getTableIndex(table, complete, error) {
        this._sqlite.getIndex(table, complete, error);
    }
    getTableIndexLength(table, complete, error) {
        this._sqlite.getNumberOfRecords(table, complete, error);
    }
}
//# sourceMappingURL=index.js.map