import { Application } from '@nativescript/core';
import { SQLiteDatabaseBase } from './sqlitedatabase.android';
function createDb(dbName, flags) {
    if (dbName === ':memory:') {
        return android.database.sqlite.SQLiteDatabase.create(flags);
    }
    if (dbName.indexOf('/') >= 0) {
        return android.database.sqlite.SQLiteDatabase.openDatabase(dbName, null, flags !== undefined
            ? flags
            : android.database.sqlite.SQLiteDatabase.CREATE_IF_NECESSARY |
                android.database.sqlite.SQLiteDatabase.NO_LOCALIZED_COLLATORS);
    }
    else {
        const activity = Application.android.foregroundActivity || Application.android.startActivity;
        return activity.openOrCreateDatabase(dbName, flags !== undefined ? flags : android.app.Activity.MODE_PRIVATE, null);
    }
}
export class SQLiteDatabase extends SQLiteDatabaseBase {
    async open() {
        if (!this.db) {
            this.db = createDb(this.filePath, this.flags);
            if (this.threading && !this.worker) {
                const Worker = require('nativescript-worker-loader!./worker');
                this.worker = new Worker();
                this.worker.onmessage = this.onWorkerMessage;
            }
        }
        return this.isOpen;
    }
}
export const openOrCreate = (filePath, options) => {
    const obj = new SQLiteDatabase(filePath, options);
    obj.open();
    return obj;
};
export const deleteDatabase = (filePath) => android.database.sqlite.SQLiteDatabase.deleteDatabase(new java.io.File(filePath));
//# sourceMappingURL=sqlite.android.js.map