export function isNothing(x) {
    return x === undefined || x === null;
}
export function arrayToNativeByteArray(val) {
    const length = val.length;
    const result = Array.create('byte', length);
    for (let i = 0; i < length; i++) {
        result[i] = val[i];
    }
    return result;
}
export function paramToString(p) {
    if (isNothing(p)) {
        return null;
    }
    if (global.isAndroid) {
        if (p instanceof java.nio.ByteBuffer) {
            return p.array();
        }
        if (p instanceof java.io.ByteArrayOutputStream) {
            return p.toByteArray();
        }
        if (p instanceof java.lang.Object) {
            return p;
        }
        if (p instanceof ArrayBuffer) {
            return arrayToNativeByteArray(new Uint8Array(p));
        }
        else if (p.buffer) {
            return arrayToNativeByteArray(p);
        }
        else if (Array.isArray(p)) {
            return arrayToNativeByteArray(p);
        }
    }
    else {
        if (p instanceof NSData) {
            return p;
        }
        if (p instanceof NSObject) {
            return p;
        }
        if (p instanceof ArrayBuffer) {
            return NSData.dataWithData(p);
        }
        if (p.buffer) {
            return NSData.dataWithData(p.buffer);
        }
        if (Array.isArray(p)) {
            return NSData.dataWithData(new Uint8Array(p).buffer);
        }
    }
    if (p.hasOwnProperty('length') && !Array.isArray(p)) {
        return p;
    }
    if (p['toString']) {
        return p['toString']();
    }
    return p;
}
export function paramsToStringArray(params) {
    if (isNothing(params)) {
        return [];
    }
    if (params instanceof Array) {
        return params.map(paramToString);
    }
    return [paramToString(params)];
}
export function throwError(msg) {
    throw new Error(`NSqlite Error: ${msg}`);
}
export function openOrCreate(filePath, flags) {
    return null;
}
export function deleteDatabase(filePath) {
    return false;
}
//# sourceMappingURL=sqlite.common.js.map