import { AbstractSqliteDriver } from '@nativescript-community/typeorm/browser/driver/sqlite-abstract/AbstractSqliteDriver';
import { DriverOptionNotSetError } from '@nativescript-community/typeorm/browser/error/DriverOptionNotSetError';
import { DriverPackageNotInstalledError } from '@nativescript-community/typeorm/browser/error/DriverPackageNotInstalledError';
import { NativescriptQueryRunner } from './NativescriptQueryRunner';
export class NativescriptDriver extends AbstractSqliteDriver {
    constructor(connection) {
        super(connection);
        this.connection = connection;
        this.options = connection.options;
        this.database = this.options.database;
        this.driver = this.options.driver;
        if (!this.options.database) {
            throw new DriverOptionNotSetError('database');
        }
        this.loadDependencies();
    }
    async disconnect() {
        this.queryRunner = undefined;
        return this.databaseConnection.close();
    }
    createQueryRunner(mode = 'master') {
        if (!this.queryRunner) {
            this.queryRunner = new NativescriptQueryRunner(this);
        }
        return this.queryRunner;
    }
    normalizeType(column) {
        if (column.type === Buffer) {
            return 'blob';
        }
        return super.normalizeType(column);
    }
    async createDatabaseConnection() {
        try {
            const db = this.sqlite.openOrCreate(this.options.database, this.options.extra ? Object.assign({}, this.options.extra) : {});
            await db.execute('PRAGMA foreign_keys = ON;', []);
            return db;
        }
        catch (err) {
            throw new Error(err);
        }
    }
    loadDependencies() {
        try {
            this.sqlite = this.options.driver || require('@akylas/nativescript-sqlite');
        }
        catch (e) {
            throw new DriverPackageNotInstalledError('Nativescript', '@akylas/nativescript-sqlite');
        }
    }
}
export class MongoDriver {
}
export class SqljsDriver {
}
export class PostgresDriver {
}
export class CockroachDriver {
}
export class MysqlDriver {
}
export class AuroraDataApiDriver {
}
export class OracleDriver {
}
export class SqlServerDriver {
}
export class BetterSqlite3Driver {
}
export class SapDriver {
}
//# sourceMappingURL=NativescriptDriver.js.map