import { QueryRunnerAlreadyReleasedError } from '@nativescript-community/typeorm/browser/error/QueryRunnerAlreadyReleasedError';
import { QueryFailedError } from '@nativescript-community/typeorm/browser/error/QueryFailedError';
import { AbstractSqliteQueryRunner } from '@nativescript-community/typeorm/browser/driver/sqlite-abstract/AbstractSqliteQueryRunner';
import { Broadcaster } from '@nativescript-community/typeorm/browser/subscriber/Broadcaster';
export class NativescriptQueryRunner extends AbstractSqliteQueryRunner {
    constructor(driver) {
        super();
        this.driver = driver;
        this.connection = driver.connection;
        this.broadcaster = new Broadcaster(this);
    }
    async query(query, parameters) {
        if (this.isReleased) {
            throw new QueryRunnerAlreadyReleasedError();
        }
        const connection = this.driver.connection;
        const isInsertQuery = query.startsWith('INSERT INTO') || query.startsWith('UPDATE');
        connection.logger.logQuery(query, parameters, this);
        try {
            const db = await this.connect();
            const queryStartTime = +new Date();
            const result = isInsertQuery
                ? await db.execute(query, parameters)
                : await db.select(query, parameters);
            const queryEndTime = +new Date();
            const queryExecutionTime = queryEndTime - queryStartTime;
            const maxQueryExecutionTime = connection.options.maxQueryExecutionTime;
            if (maxQueryExecutionTime && queryExecutionTime > maxQueryExecutionTime) {
                connection.logger.logQuerySlow(queryExecutionTime, query, parameters, this);
            }
            return result;
        }
        catch (err) {
            connection.logger.logQueryError(err, query, parameters, this);
            throw new QueryFailedError(query, parameters, err);
        }
    }
    parametrize(objectLiteral, startIndex = 0) {
        return Object.keys(objectLiteral).map((key, index) => `"${key}"` + '=?');
    }
}
//# sourceMappingURL=NativescriptQueryRunner.js.map