"use strict";
const converter_1 = require("./converter");
const data_provider_1 = require("./data.provider");
function getPlatformData(platformData, projectData, platform, injector) {
    if (!platformData) {
        const platformsData = injector.resolve('platformsData');
        platformData = platformsData.getPlatformData(platform, projectData);
    }
    return platformData;
}
const hookArgReader = (args) => {
    if (typeof args !== 'string') {
        return Object.keys(args)[0];
    }
    else {
        return args;
    }
};
module.exports = function (androidResourcesMigrationService, logger, projectData, injector, hookArgs) {
    const platformName = ((hookArgs && hookArgs.platformData && hookArgs.platformData.normalizedPlatformName) || (hookArgs.checkForChangesOpts && hookArgs.checkForChangesOpts.platform) || '').toLowerCase();
    projectData = hookArgs && (hookArgs.projectData || (hookArgs.checkForChangesOpts && hookArgs.checkForChangesOpts.projectData));
    const platformData = getPlatformData(hookArgs && hookArgs.platformData, projectData, platformName, injector);
    let converter;
    const dataProvider = new data_provider_1.DataProvider(logger, projectData);
    const environmentName = hookArgs.prepareData.env.use ? hookArgReader(hookArgs.prepareData.env.use) : '';
    if (platformName === 'android') {
        converter = new converter_1.ConverterAndroid(dataProvider, logger, platformData, projectData, androidResourcesMigrationService, environmentName);
    }
    else if (platformName === 'ios') {
        converter = new converter_1.ConverterIOS(dataProvider, logger, platformData, projectData, environmentName);
    }
    else {
        logger.warn(`Platform '${platformName}' isn't supported: skipping localization`);
        return;
    }
    converter.run();
};
