"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValue = void 0;
const index_1 = require("../index");
const log_1 = require("./log");
function getCLILib() {
    if (!index_1.env.nativescriptLibPath) {
        log_1.warnOnce('getCLILib', `
			Cannot find NativeScript CLI path. Make sure --env.nativescriptLibPath is passed
		`);
        return false;
    }
    return require(index_1.env.nativescriptLibPath);
}
/**
 * Utility to get a value from the nativescript.config.ts file.
 *
 * @param {string} key The key to get from the config. Supports dot-notation.
 * @param defaultValue The fallback value if the key is not set in the config.
 */
function getValue(key, defaultValue) {
    const lib = getCLILib();
    if (!lib) {
        return defaultValue;
    }
    return lib.projectConfigService.getValue(key, defaultValue);
}
exports.getValue = getValue;
//# sourceMappingURL=config.js.map