import { Utils } from "@nativescript/core";
import { startButNotStopped, stopButNotStarted, SensorDelay } from "./common";
export { SensorDelay };
const baseAcceleration = -9.81;
let sensorListener;
let sensorManager;
let accelerometerSensor;
function getNativeDelay(options) {
    if (!options || !options.sensorDelay) {
        return android.hardware.SensorManager.SENSOR_DELAY_NORMAL;
    }
    switch (options.sensorDelay) {
        case SensorDelay.Normal:
            return android.hardware.SensorManager.SENSOR_DELAY_NORMAL;
        case SensorDelay.Game:
            return android.hardware.SensorManager.SENSOR_DELAY_GAME;
        case SensorDelay.UI:
            return android.hardware.SensorManager.SENSOR_DELAY_UI;
        case SensorDelay.Fastest:
            return android.hardware.SensorManager.SENSOR_DELAY_FASTEST;
    }
}
export function startAccelerometerUpdates(callback, options) {
    if (isListening()) {
        console.log(startButNotStopped);
        stopAccelerometerUpdates();
    }
    const wrappedCallback = zonedCallback(callback);
    const context = Utils.ad.getApplicationContext();
    if (!context) {
        throw Error("Could not get Android application context.");
    }
    if (!sensorManager) {
        sensorManager = context.getSystemService(android.content.Context.SENSOR_SERVICE);
        if (!sensorManager) {
            throw Error("Could not initialize SensorManager.");
        }
    }
    if (!accelerometerSensor) {
        accelerometerSensor = sensorManager.getDefaultSensor(android.hardware.Sensor.TYPE_ACCELEROMETER);
        if (!accelerometerSensor) {
            throw Error("Could get accelerometer sensor.");
        }
    }
    sensorListener = new android.hardware.SensorEventListener({
        onAccuracyChanged: (sensor, accuracy) => {
        },
        onSensorChanged: (event) => {
            wrappedCallback({
                x: event.values[0] / baseAcceleration,
                y: event.values[1] / baseAcceleration,
                z: event.values[2] / baseAcceleration
            });
        }
    });
    const nativeDelay = getNativeDelay(options);
    sensorManager.registerListener(sensorListener, accelerometerSensor, nativeDelay);
}
export function stopAccelerometerUpdates() {
    if (sensorListener) {
        sensorManager.unregisterListener(sensorListener);
        sensorListener = undefined;
    }
    else {
        console.log(stopButNotStarted);
    }
}
export function isListening() {
    return !!sensorListener;
}
//# sourceMappingURL=index.android.js.map